import { IConnector } from "../../common/connector";
import { DatabaseType } from "prisma-datamodel";
import { RelationalIntrospectionResult } from "./relationalIntrospectionResult";
export interface IInternalIndexInfo {
    tableName: string;
    name: string;
    fields: string[];
    unique: boolean;
    isPrimaryKey: boolean;
}
export interface IInternalEnumInfo {
    name: string;
    values: string[];
}
export declare abstract class RelationalConnector implements IConnector {
    abstract getDatabaseType(): DatabaseType;
    protected abstract createIntrospectionResult(models: ITable[], relations: ITableRelation[], enums: IEnum[]): RelationalIntrospectionResult;
    protected abstract query(query: string, params?: any[]): Promise<any[]>;
    /**
     * Column comments are DB specific
     */
    protected abstract queryColumnComment(schemaName: string, tableName: string, columnName: string): Promise<string | null>;
    /**
     * Indices are DB specific
     */
    protected abstract queryIndices(schemaName: string, tableName: string): Promise<IInternalIndexInfo[]>;
    protected abstract queryEnums(schemaName: string): Promise<IEnum[]>;
    introspect(schema: string): Promise<RelationalIntrospectionResult>;
    listEnums(schemaName: string): Promise<IEnum[]>;
    /**
     * All queries below use the standardized information_schema table.
     */
    listSchemas(): Promise<string[]>;
    protected listModels(schemaName: string): Promise<ITable[]>;
    protected queryTables(schemaName: string): Promise<string[]>;
    protected queryColumns(schemaName: string, tableName: string): Promise<{
        name: string;
        type: string;
        isList: boolean;
        readOnly: boolean;
        isUnique: boolean;
        defaultValue: string;
        isNullable: boolean;
        comment: string | null;
    }[]>;
    protected listRelations(schemaName: string): Promise<ITableRelation[]>;
}
export interface IEnum {
    name: string;
    values: string[];
}
export interface ITable {
    name: string;
    columns: IColumn[];
    indices: IIndex[];
    primaryKey: IIndex | null;
}
export interface IColumn {
    name: string;
    isUnique: boolean;
    defaultValue: any;
    type: string;
    comment: string | null;
    isNullable: boolean;
    isList: boolean;
}
export interface ITableRelation {
    sourceTable: string;
    targetTable: string;
    sourceColumn: string;
    targetColumn: string;
}
export interface IIndex {
    name: string;
    fields: string[];
    unique: boolean;
}
